@@ -255,7 +255,7 @@ class UnattendedUpgradesCache(apt.Cache):
         pins = []  # type: List[PkgPin]
         for regex in regexps:
             if python_regex_is_posix(regex):
-                pins.append(PkgPin('/' + regex + '/', priority))
+                pins.append(PkgPin('/^' + regex + '/', priority))
             else:
                 # Python regex is not also an equivalent POSIX regexp.
                 # This is expected to be rare. Go through all the package names
@@ -342,7 +342,7 @@ class UnattendedUpgradesCache(apt.Cache):
             # Only adjust to lower versions to avoid flipping back and forth
             # and to avoid picking a newer version, not selected by apt.
             # This helps avoiding upgrades to experimental's packages.
-            if new_cand < pkg.candidate:
+            if pkg.candidate is not None and new_cand < pkg.candidate:
                 logging.debug("adjusting candidate version: %s" % new_cand)
                 pkg.candidate = new_cand
                 return True
@@ -377,15 +377,15 @@ class UnattendedUpgradesCache(apt.Cache):
         """
         new_pkgs_to_adjust = []  # List[str]
 
-        if pkg._pkg.selected_state == apt_pkg.SELSTATE_HOLD:
-            logging.debug("pkg %s is on hold" % pkg.name)
+        if not is_pkg_change_allowed(pkg, self.blacklist, self.whitelist,
+                                     self.strict_whitelist):
             return
 
         if function == apt.package.Package.mark_upgrade \
                 and not pkg.is_upgradable:
             if not apt_pkg.config.find_b("Unattended-Upgrade::Allow-downgrade",
                                          False):
-                raise NoAllowedOriginError
+                return
             else:
                 function = apt.package.Package.mark_install
         marking_succeeded = self.call_checked(function, pkg, **kwargs)
@@ -841,7 +841,7 @@ def match_whitelist_string(whitelist, origin):
 def python_regex_is_posix(expression):
     # type: (str) -> bool
     """ Returns if the Python regex is also an equivalent POSIX regex """
-    return re.match("^[a-zA-Z0-9\\^\\$\\+\\.]*$", expression) is None
+    return re.match("^[a-zA-Z0-9\\^\\$\\+\\.]*$", expression) is not None
 
 
 def cache_commit(cache,           # type: apt.Cache
@@ -1508,7 +1508,7 @@ def send_summary_mail(pkgs,                 # type: List[str]
 
 
 def do_install(cache,             # type: UnattendedUpgradesCache
-               pkgs_to_upgrade,   # type: List[apt.Package]
+               pkgs_to_upgrade,   # type: List[str]
                options,           # type: Options
                logfile_dpkg,      # type: str
                ):
@@ -1518,18 +1518,15 @@ def do_install(cache,             # type: UnattendedUpgradesCache
 
     logging.info(_("Writing dpkg log to %s"), logfile_dpkg)
 
-    marked_delete = [pkg for pkg in cache.get_changes() if pkg.marked_delete]
-    if marked_delete:
-        raise AssertionError(
-            "Internal error. The following packages are marked for "
-            "removal:%s" % " ".join([pkg.name for pkg in marked_delete]))
+    if cache.get_changes():
+        cache.clear()
 
     pkg_install_success = False
     try:
         if options.minimal_upgrade_steps:
             # try upgrade all "pkgs" in minimal steps
             pkg_install_success = upgrade_in_minimal_steps(
-                cache, [pkg.name for pkg in pkgs_to_upgrade],
+                cache, pkgs_to_upgrade,
                 logfile_dpkg,
                 options.verbose or options.debug)
         else:
@@ -2024,8 +2021,9 @@ def main(options, rootdir="/"):
 
 
 def mark_pkgs_to_upgrade(cache, pkgs_to_upgrade):
-    # type (apt.Cache, List[apt.Package]) -> None
-    for pkg in pkgs_to_upgrade:
+    # type (apt.Cache, List[str]) -> None
+    for pkg_name in pkgs_to_upgrade:
+        pkg = cache[pkg_name]
         if pkg.is_upgradable \
            or (pkg.is_installed
                and pkg.candidate.version != pkg.installed.version) \
@@ -2165,7 +2163,7 @@ def run(options,             # type: Options
     os.nice(old_priority - os.nice(0))
 
     # download what looks good
-    mark_pkgs_to_upgrade(cache, pkgs_to_upgrade)
+    mark_pkgs_to_upgrade(cache, pkgs)
 
     if options.debug:
         fetcher = apt_pkg.Acquire(apt.progress.text.AcquireProgress())
@@ -2258,6 +2256,8 @@ def run(options,             # type: Options
                       % conffile_blacklist)
         # find out about the packages that are upgradable (in a allowed_origin)
         if len(conffile_blacklist) > 0:
+            for regex in conffile_blacklist:
+                cache.blacklist.append(regex)
             cache.apply_pinning(cache.pinning_from_regex_list(
                 conffile_blacklist, NEVER_PIN))  # type: ignore
             old_pkgs_to_upgrade = pkgs_to_upgrade[:]
@@ -2357,7 +2357,7 @@ def run(options,             # type: Options
     if len(pkgs_to_upgrade) > 0:
         # do install
         pkg_install_success = do_install(cache,
-                                         pkgs_to_upgrade,
+                                         pkgs,
                                          options,
                                          logfile_dpkg)
     # Was the overall run succesful: only if everything installed
